<?php

namespace ID3\DLP;

use Error;

require_once(PLUGIN_PATH . '/core/class.rest.php');
require_once(PLUGIN_PATH . '/core/class.slave.php');

class Main{

    const CONFIG_PAGE_KEY = 'id3-dlp';
    const CONFIG_MENU_LOCATION = 'settings';

    public $rest;

    function __construct(){
        
        $this->register_actions();
        $this->register_rest();
        $this->register_post_metaboxes();

        
    }

    function register_actions() {
        add_action('admin_init', [$this, 'register_admin_init']);
        add_action('admin_menu', [$this, 'register_admin_menu']);
        add_action('admin_enqueue_scripts', [$this, 'register_admin_enqueue_scripts']);

        // add action for ID-3 Blog update, in case of a DLP Chain (This installation gets articles from another DLP and relay them to slave sites)
        add_action('id3-blog/sync/new-articles', [$this, 'trigger_slave_updates']);
    }

    static function log($message){
        $log  = $_SERVER['REMOTE_ADDR'].' - '. date("h:i:s") .' - '.$message."\n";

        //Save string to log, use FILE_APPEND to append.
        file_put_contents(PLUGIN_PATH . '/logs/log_'. date("dmY") .'.log', $log, FILE_APPEND);
    }

    //#region Admin

    function register_admin_init(){
        
    }

    function register_admin_menu(){
        add_options_page(
            __('ID-3 DLP'),
            __('ID-3 Delivery Service'),
            'manage_options',
            self::CONFIG_PAGE_KEY,
            [$this, 'render_admin_page']
      );
    }

    function register_admin_enqueue_scripts($page){
        $ressource_location = 'settings_page_';
        if(self::CONFIG_MENU_LOCATION == 'root' || is_network_admin()){
        $ressource_location = 'toplevel_page_';
        }

        if($page == $ressource_location  . self::CONFIG_PAGE_KEY){
            wp_enqueue_media();
            wp_enqueue_style( 'fontawesome5', plugins_url('/styles/fa/all.min.css', PLUGIN) );
            wp_enqueue_style( 'id3dlp-style', plugins_url('/styles/admin.min.css', PLUGIN) );
        }
    }

    function render_admin_page(){
        include_once PLUGIN_DIR . 'views/admin.php';
    }

    //#endregion

    function register_rest(){
        $this->rest = new \ID3\DLP\Rest();
    }

    function register_post_metaboxes(){
        // lang
        add_action('load-post.php', [$this, 'lang_metabox_setup']);
        add_action('load-post-new.php', [$this, 'lang_metabox_setup']);
    }

    function lang_metabox_setup(){
        add_action( 'add_meta_boxes', [$this, 'add_lang_metabox'] );
        add_action( 'save_post', [$this, 'save_lang_metabox_data'] );

        add_action( 'post_updated_messages', [$this, 'trigger_slave_updates'] );
    }

    function add_lang_metabox(){
        add_meta_box(
            'id3-dlp-post-lang',      // Unique ID
            esc_html__( 'Language' ),    // Title
            [$this, 'lang_meta_box'],   // Callback function
            'post',         // Admin page (or post type)
            'side',         // Context
            'default'         // Priority
          );
    }

    function lang_meta_box($post){
        $lang = get_post_meta( $post->ID, 'id3-dlp-post-lang', true );

        $choices = [
            'fr' => 'Français',
            'en' => 'English',
            'es' => 'Español'   
        ];
        ?>

        <?php wp_nonce_field( basename( __FILE__ ), 'id3_dlp_lang_meta_box_nonce' ); ?>
      
        <div class="components-panel__row">
          <span><?php _e( "Language of this article", 'id3-dlp' ); ?></span>
          
          <div class="components-base-control">
            <select name="id3-dlp-post-lang" id="id3-dlp-post-lang" >
                <?php
                foreach ($choices as $key => $value) {
                    echo('<option value="' . $key . '"');
                    if($key === $lang){
                        echo(' selected');
                    }
                    echo('>' . $value .'</option>');
                }
                ?>
            </select>
          </div>
        </div>
      <?php
    }

    function save_lang_metabox_data( $post_id ){
        // verify taxonomies meta box nonce
        if ( !isset( $_POST['id3_dlp_lang_meta_box_nonce'] ) ){
            return;
        }
    
        // return if autosave
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ){
            return;
        }
    
        // Check the user's permissions.
        if ( ! current_user_can( 'edit_post', $post_id ) ){
            return;
        }
            
        // store custom fields values
        if( isset( $_POST['id3-dlp-post-lang'] ) ){
            
            $lang = $_POST['id3-dlp-post-lang'];
    
            // save data
            update_post_meta( $post_id, 'id3-dlp-post-lang', $lang );
        }else{
            // delete data
            delete_post_meta( $post_id, 'id3-dlp-post-lang' );
        }
    }

    /**
     * Load slaves and call their sync method
     * @param mixed $post_id 
     * @return void 
     */
    function trigger_slave_updates($post_id=''){

        self::log("{$post_id} triggered a slave update");
        $slaves = $this->get_slaves();

        foreach ($slaves as $slave_id) {
            $slave = new Slave($slave_id);
            if($slave->active){
                $slave->call_remote_sync();
            }
        }
    }

    function get_slaves(){
        $slaves = get_transient('id3-dlp-slaves');
        if($slaves != null){
            $slaves = json_decode($slaves);
        }
        else{
            $slaves = [];
        }

        return $slaves;
    }

    
    function update_slave_data($slave_id, $infos){
        $slaves = $this->get_slaves();
        if(!in_array($slave_id, $slaves)){
            $slaves[] = $slave_id;
            set_transient('id3-dlp-slaves', json_encode($slaves));
        }
        
        set_transient('id3-dlp-' . sanitize_title($slave_id), $infos);
        
    }

    static function is_plugin_active($plugin_name){
        $plugins = get_option( 'active_plugins', [] );
        foreach ($plugins as $value) {
            if(strpos($value, $plugin_name) !== false) return true;
        }

        if ( ! is_multisite() ) return false;


        $plugins = get_site_option( 'active_sitewide_plugins' );
        foreach ($plugins as $value) {
            if(strpos($value, $plugin_name) !== false) return true;
        }
        return false;
    }
}