<?php

namespace ID3\DLP;

use AsyncCurl;

class Slave{
    private $_key = '';

    public $id;
    public $active = true;
    public $filters = [];
    public $last_sync = null;
    public $unreachable = false;
    public $last_update_signal_sent = null;
    public $error_message = null;
    public $compare_date = "post_modified";

    public function __construct($id=''){
        $this->id = $id;
        $this->_key = 'id3-dlp-' . sanitize_title($id);

    }

    public function load(){
        if($this->_key == '') return null;

        $isNew = true;
        $data = get_transient($this->_key);
        
        if($data != null){
            $data = json_decode($data);
            $this->active = $data->active;
            $this->filters = $data->filters;
            $this->last_sync = $data->last_sync;
            $this->last_update_signal_sent = $data->last_update_signal_sent;
            $this->error_message = $data->error_message;
            $this->unreachable = $data->unreachable;

            if(strpos($this->id,'localhost')!==false){
                $this->unreachable = true;
                $this->error_message = 'Local server not accessible from here';
            }
            $isNew = false;
        }

        if(!$this->unreachable){
            if(!isset($this->last_sync) || $this->last_sync == null){     
                $this->last_sync = Date('Y-m-d H:i:s e');
            }
        }

        return $isNew;
    }

    public function set_filters($value){
        if(!is_object($value)){ $value = json_decode(json_encode($value)); }

        $this->filters = $value;
    }

    public function get_post_query(){
        $queryOptions = [
			'posts_per_page' => '-1',
            'post_type' => 'post',
            'post_status' => ['publish','future'],
            //'suppress_filters' => true,
            'date_query' => array(
                array(
                    'column'  => $this->compare_date,
                    'after'   => $this->last_sync
                )
            ),
        ];

        if($this->filters != null){
            $currentLocale = apply_filters( 'wpml_current_language', substr(get_locale(),0,2) );
            $subscriptionFilters = $this->getFilters('subscriptions'); //array_values(array_filter($this->filters,function($f){ return !$f->all && $f->type == 'subscriptions';}));
            $exceptionFilters = $this->getFilters('exceptions'); // array_values(array_filter($this->filters,function($f){ return $f->type == 'exceptions';}));
            
            // Get all "take" categories
            if(count($subscriptionFilters)>0){
                
                $parent_categories  = array_reduce($subscriptionFilters, function($result, $cur){
                            return array_merge($result,$cur->list);
                            return $result;
                        },[]);

                $root_categories = $this->getRootCategoriesFromFilters($parent_categories);

                $queryOptions['tax_query'] = [
                        [
                          'taxonomy'         => 'category', 
                          'field'            => 'term_id', 
                          'include_children' => true,
                          'operator'         => 'IN',
                          'terms'            => $root_categories, 
                        ]
                    ];
            }

            if(count($exceptionFilters)>0){
                $queryOptions['category__not_in'] = array_reduce($exceptionFilters, function($result, $cur){
                        return array_merge($result,$cur->list);
                        return $result;
                    },[]);
            }
        }

        //error_log('Articles QUERY for: ' . $currentLocale . ' in categories ' . json_encode($parent_categories));
        //error_log(json_encode($queryOptions));

        return new \WP_Query($queryOptions);
    }

    public function getNewArticles(){
        // remove image url filtering in case of delivery relay configuration
        add_filter('id3-blog/allow-image-src-filtering', function() { return false; });

        // Get Post Query from slave    
        
        $the_query = $this->get_post_query();
        $rootCategories = $this->getRootCategories();

        $posts = $the_query->posts;
        foreach($posts as $post) {
            // meta to post infos
            $post->lang = $this->getPostLocale($post->ID);
            $post->categories = array_map(function($cat){ return $cat->cat_ID; }, get_the_category($post->ID));
            $post->categoryTree = [];
            foreach ($rootCategories as $rootCat) {
                $post->categoryTree[$rootCat] = [];
                
                foreach ($post->categories as $subCat) {
                    if(cat_is_ancestor_of($rootCat, $subCat)){
                        $post->categoryTree[$rootCat][] = $subCat;
                    }
                }
            }

            if(has_post_thumbnail($post->ID)){

                $image = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), 'single-post-thumbnail' );
                
                $post->feature_image = $image[0];    
            }
        }

        return $posts;
    }

    public function getPostLocale($post_id){
        // Polylang
        if(function_exists('pll_set_post_language')){
            return pll_get_post_language($post_id);
        }

        // WPML
        if(Main::is_plugin_active("sitepress")){
            $langDetails = apply_filters( 'wpml_post_language_details', NULL, $post_id ) ;
            return $langDetails['language_code'];
        }

        return get_post_meta($post_id, 'id3-dlp-post-lang',true);
    }

    public function getFilters($type){
        $result = [];
        foreach ($this->filters as $filter) {
            if($filter->type != $type) continue;
            if(is_bool($filter->all) && $filter->all === true) continue;

            $result[] = $filter;
        }

        // error_log('getFilters(' . $type . ')  => ' . json_encode($result));

        return $result;
    }

    public function applyArticleFilters($articles){
        $subscriptionFilters = $this->getFilters('subscriptions');//array_values(array_filter($this->filters,function($f){ return !$f->all && $f->type == 'subscriptions';}));
        

        // Bail out conditions
        // when no filters are set
        if(count($subscriptionFilters)==0) {
            return [
                'infos'=> 'no filter',
                'list' => $articles
            ];
        }

        // Get all root categories
        $rootCategories = $this->getRootCategories();
        // Get all subscription to categories for this slave
        $subscribedCategories  = array_reduce($subscriptionFilters, function($result, $cur){
            return array_merge($result,$cur->list);
            return $result;
        },[]);

        // Build the filter category tree based on the root and subscription
        $filterCategoryTree = [];
        forEach($rootCategories as $rootCat){
            $filterCategoryTree[$rootCat] = [];
            // loop through subscriptions and add on those who are child of the current root category
            foreach($subscribedCategories as $childcat){
                if(cat_is_ancestor_of($rootCat, $childcat)){
                    $filterCategoryTree[$rootCat][] = $childcat;
                };
            }
        }

        // Build the final filtered list of article
        $filteredList = [];
        $breakingPoints = [];
        foreach ($articles as $post){
            $breakingPoint = null;

            // Loop through all branches of the post category tree (build earlier)
            foreach($post->categoryTree as $key => $branch){

                if(!isset($filterCategoryTree[$key])){ // This should not happen, but check it anyway. Just in case
                    $breakingPoint = "Branch {$key} missing";
                    break;
                }
                // Automatically add this branch if no child has been specified
                else if(count($branch)>0){
                    // Automatically add this branch if no child has been specified in the matching filter tree branch
                    if(count($filterCategoryTree[$key])>0){
                        $branchCatCount = 0;
                        // Count the number of subcategory filter that match this post
                        foreach($filterCategoryTree[$key] as $subCat){
                            if(in_array($subCat, $branch)){
                                $branchCatCount+=1;
                                break;
                            }
                        }
                        // When the branch count no subcategory filter, it means that this post must be filter out
                        if($branchCatCount == 0){
                            $breakingPoint = "Branch {$key} couldn't find any of " . json_encode($filterCategoryTree[$key]) . ' in ' . json_encode($branch);
                        }
                    }
                }
            }


            // Add the post since there's no breaking point
            if($breakingPoint === null || count($breakingPoints) == 0){
                unset($post->categoryTree);
                $filteredList[] = $post;
            }
            else{
                // log post's breaking points
                $breakingPoints[] = ['id' => $post->ID, 'break' => $breakingPoint];
            }
        }

        
        return [
            'list' => $filteredList,
            'infos' => [
                'filterCatTree' => $filterCategoryTree,
                'breakingPoints' => $breakingPoints
            ]
        ];
    }

    public function loadArticlesExtraMeta(&$articles){

        foreach ($articles as &$post) {
            $metas = get_post_meta($post->ID);
            if($metas !== false){
                $post->metas = [];//$metas;
                foreach($metas as $key => $meta){
                    // This is a Yoast Meta
                    if(strpos($key, '_yoast_wpseo_') !== false){
                        if($key == '_yoast_wpseo_primary_category') continue;
                        $post->metas[] = ['key' => $key, 'value' => $meta[0]];
                    }
                }
            }
        }

        return $articles;
    }

    public function getRootCategories(){
        $categories = get_categories([
            'hide_empty'=> false,
            'parent'    => 0,
            //'suppress_filters' => true
        ]);
        $result = [];
        foreach ($categories as $cat) {
            $result[] = $cat->term_id;
        }

        return $result;
    }

    public function getRootCategoriesFromFilters($filterCategories){
        $result = [];
        $rootCategories = $this->getRootCategories();
        //error_log('rootCategories -> ' . json_encode($rootCategories));

        $rootWithFilters = [];

        foreach ($rootCategories as $rootCat) {
            foreach($filterCategories as $subCat){
                if($rootCat == $subCat){
                    $result[] = $rootCat;
                }
                else if (cat_is_ancestor_of($rootCat, $subCat)){
                    if(!in_array($rootCat, $result)){
                        $result[] = $rootCat;
                        $rootWithFilters[] = $rootCat;
                    }
                }
            }
        }

        foreach ($rootCategories as $rootCat) {
            if(!in_array($rootCat, $rootWithFilters)){
                if(!empty(get_term_children($rootCat,'category'))){
                    $result[] = $rootCat;
                }
            }
        }

        return $result;
    }


    

    public function delete(){
        $list = $this->list();
        $newList = array_values(array_filter($list, function($entry){
            return $entry != $this->id;
        }));

        set_transient('id3-dlp-slaves', json_encode($newList));
        delete_transient($this->_key);
        return $newList;
    }


    public function list(){
        $slaves = get_transient('id3-dlp-slaves');
        if($slaves != null){
            $slaves = json_decode($slaves,true);
            if(count(array_keys($slaves))>0){
                $slaves = array_values($slaves);
            }
        }
        else{
            $slaves = [];
        }

        return $slaves;
    }

    public function save_to_list(){
        $slaves = $this->list();

        if(!in_array($this->id, $slaves)){
            $slaves[] = $this->id;
            set_transient('id3-dlp-slaves', json_encode($slaves));
        }
    }

    public function call_remote_sync(){
        if(strpos($this->id,'localhost')!==false) return;
        // require_once('class.async.curl.php');
        $url = 'https://' . $this->id . '/wp-json/id3-rest/blog/signal_update';
        Main::log("sending signal update to {$this->id}");
        //open connection
        $ch = curl_init();

        //set the url, number of POST vars, POST data
        curl_setopt($ch,CURLOPT_URL, $url);
        curl_setopt($ch,CURLOPT_POST, true);
        
        //execute post
        $result = curl_exec($ch);
        $error = curl_error($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        //close connection
        curl_close($ch);

        // $query = new AsyncCurl();
        // $query->set_param ($url);
        // $query->send();

        

        Main::log("{$this->id} respond with {$httpCode}");
        if($error != null){
            Main::log("{$this->id} response: ERR:{$error}");
        }
        else{
            Main::log("{$this->id} response: {$result}");
        }

        if($httpCode == 200){
            $this->unreachable = false;
            $this->last_update_signal_sent = Date('Y-m-d H:i:s e');
            $this->update();
        }
        else{
            //$this->last_update_signal_sent = null;
            $this->unreachable = true;
            $this->error_message = $error;
            $this->update();
        }
    }

    public function update(){
        $this->save_to_list();
        set_transient($this->_key, json_encode($this));
    }
}